@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name$
-// @(#) $Id: mcblock.cc 9418 2017-06-04 03:08:30Z dwm $
+// @(#) $Id: mcblock.cc 12091 2022-11-29 05:05:27Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2015
 //  All rights reserved.
@@ -51,7 +51,7 @@ extern "C" {
 #include "DwmOptArgs.hh"
 #include "DwmSocket.hh"
 
-#include "DwmAuthPeerAuthenticator.hh"
+#include "DwmCredencePeer.hh"
 #include "DwmRDAPRequestMessage.hh"
 #include "DwmRDAPResponseMessage.hh"
 #include "DwmMcBlockAuthLogParser.hh"
@@ -63,7 +63,7 @@ extern "C" {
 using namespace Dwm;
 using namespace std;
 
-static const Dwm::CvsTag cvstag("@(#) $Name$ $Id: mcblock.cc 9418 2017-06-04 03:08:30Z dwm $");
+static const Dwm::CvsTag cvstag("@(#) $Name$ $Id: mcblock.cc 12091 2022-11-29 05:05:27Z dwm $");
 
 //----------------------------------------------------------------------------
 //!  
@@ -121,31 +121,24 @@ static void ApplyAddRule(McBlock::DbEntry & dbEntry, const Ipv4Prefix & pfx)
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-bool GetRDAPConnection(Socket & s, string & agreedKey)
+bool GetRDAPConnection(Credence::Peer & peer)
 {
   bool   rc = false;
   char  *rhost = getenv("RDAPDHOST");
   if (rhost) {
-    struct hostent  *hostEntry = gethostbyname2(rhost, AF_INET);
-    if (s.Open(PF_INET, SOCK_STREAM, 0)) {
-      struct sockaddr_in  servAddr;
-      memset(&servAddr, 0, sizeof(servAddr));
-      servAddr.sin_len = sizeof(servAddr);
-      servAddr.sin_family = AF_INET;
-      servAddr.sin_port = htons(6363);
-      servAddr.sin_addr.s_addr = *(in_addr_t *)hostEntry->h_addr_list[0];
-      if (s.Connect(servAddr)) {
-        int  noDelay = 1;
-        // s.Setsockopt(IPPROTO_TCP, TCP_NODELAY, &noDelay, sizeof(noDelay));
-        Auth::PeerAuthenticator  peer("", "");
-        string  theirId;
-        if (peer.Authenticate(s, theirId, agreedKey)) {
-          rc = true;
-        }
-        else {
-          s.Close();
-        }
+    if (peer.Connect(rhost, 6363)) {
+      Credence::KeyStash   keyStash;
+      Credence::KnownKeys  knownKeys;
+      if (peer.Authenticate(keyStash, knownKeys)) {
+        rc = true;
       }
+      else {
+        cerr << "Failed to authenticate with RDAPHOST " << rhost << '\n';
+        peer.Disconnect();
+      }
+    }
+    else {
+      cerr << "Failed to connect to RDAPHOST " << rhost << '\n';
     }
   }
   return rc;
@@ -366,20 +359,19 @@ static bool ForeignRegistry(const Ipv4Address & addr)
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-bool RDAPGet(Socket & s, const string & agreedKey,
-             const Ipv4Address & addr, Ipv4Prefix & pfx,
-             string & ccode)
+bool RDAPGet(Credence::Peer & peer, const Ipv4Address & addr,
+             Ipv4Prefix & pfx, string & ccode)
 {
   bool  rc = false;
   vector<Ipv4Address>   addrs;
   addrs.push_back(addr);
   RDAP::RequestMessage  req(addrs);
-  if (req.Write(s, agreedKey)) {
+  if (peer.Send(req)) {
     RDAP::ResponseMessage  resp;
-    if (resp.Read(s, agreedKey)) {
-      if ((! resp.Json().empty()) && (resp.Json().isArray())) {
-        pfx = Ipv4Prefix(resp.Json()[0]["prefix"].asString());
-        ccode = resp.Json()[0]["country"].asString();
+    if (peer.Receive(resp)) {
+      if ((! resp.Json().empty()) && (resp.Json().is_array())) {
+        pfx = Ipv4Prefix(resp.Json()[0]["prefix"].get<string>());
+        ccode = resp.Json()[0]["country"].get<string>();
         rc = true;
       }
     }
@@ -430,9 +422,8 @@ static bool ShowAuthLogEntries(const string & dbFilename,
   if (! logEntries.empty()) {
     McBlock::Db  db;
     db.Load(dbFilename);
-    Socket  rdapSock;
-    string  agreedKey;
-    bool    haveRdap = GetRDAPConnection(rdapSock, agreedKey);
+    Credence::Peer  peer;
+    bool    haveRdap = GetRDAPConnection(peer);
     for (auto logEntry : logEntries) {
       if ((logEntry.second >= threshold)
           || (ForeignRegistry(logEntry.first))) {
@@ -450,7 +441,7 @@ static bool ShowAuthLogEntries(const string & dbFilename,
           if (haveRdap) {
             string      ccode;
             Ipv4Prefix  pfx;
-            if (RDAPGet(rdapSock, agreedKey, logEntry.first, pfx, ccode)) {
+            if (RDAPGet(peer, logEntry.first, pfx, ccode)) {
               newEntry = McBlock::DbEntry(Ipv4Prefix(logEntry.first, 24),
                                           ti, "", ccode);
               ApplyAddRule(newEntry, pfx);
@@ -598,15 +589,14 @@ static bool AddAuthLogEntries(const string & dbFilename,
   if (! logEntries.empty()) {
     McBlock::Db  db;
     db.Load(dbFilename);
-    Socket  rdapSock;
-    string  agreedKey;
-    bool    haveRdap = GetRDAPConnection(rdapSock, agreedKey);
+    Credence::Peer  peer;
+    bool    haveRdap = GetRDAPConnection(peer);
     for (auto logEntry : logEntries) {
       if ((logEntry.second >= threshold) || ForeignRegistry(logEntry.first)) {
         if (haveRdap) {
           string      ccode;
           Ipv4Prefix  pfx;
-          if (RDAPGet(rdapSock, agreedKey, logEntry.first, pfx, ccode)) {
+          if (RDAPGet(peer, logEntry.first, pfx, ccode)) {
             Dwm::TimeValue  startTime(true);
             Dwm::TimeValue  endTime;
             endTime.Set(startTime.Secs() + (days * 24 * 60 * 60) + 1, 0);
