@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: mcblockc.cc 12101 2022-12-04 01:17:28Z dwm $
+// @(#) $Id: mcblockc.cc 12351 2024-05-03 09:12:30Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -65,7 +65,7 @@ extern "C" {
 #include "DwmMcBlockRequestMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockc/mcblockc.cc 12101 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.6/apps/mcblockc/mcblockc.cc 12351 $");
 
 using namespace std;
 using namespace Dwm;
@@ -143,7 +143,9 @@ bool Connect(const string & host, Credence::Peer & peer)
 static void Usage(const string & argv0)
 {
   cerr << "Usage: " << argv0 << " [-h host] activate tableName prefix(es)\n"
+       << "       " << argv0 << " [-h host] add tableName country days prefix(es)\n"
        << "       " << argv0 << " [-h host] deactivate tableName prefix(es)\n"
+       << "       " << argv0 << " [-h host] delete tableName prefix(es)\n"
        << "       " << argv0 << " [-h host] getactive tableName\n"
        << "       " << argv0 << " [-h host] getaddrules tableName\n"
        << "       " << argv0 << " [-h host] loghit tableName ipv4addr\n"
@@ -154,6 +156,43 @@ static void Usage(const string & argv0)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool AddPrefixes(Credence::Peer & peer, const string & table,
+                        const string & country, uint32_t days,
+                        const vector<Ipv4Prefix> & prefixes)
+{
+  bool  rc = false;
+  McBlock::ResponseMessage  resp;
+  McBlock::RequestMessage   req(McBlock::AddReq(table, country, days, prefixes));
+  if (RequestResponse(peer, req, resp)) {
+    if (! resp.Json().empty()) {
+      cout << resp.Json().dump(2) << '\n';
+      rc = true;
+    }
+  }
+  return rc;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool DeletePrefixes(Credence::Peer & peer, const string & table,
+                           const vector<Ipv4Prefix> & prefixes)
+{
+  bool  rc = false;
+  McBlock::ResponseMessage  resp;
+  McBlock::RequestMessage   req(McBlock::DeleteReq(table, prefixes));
+  if (RequestResponse(peer, req, resp)) {
+    if (! resp.Json().empty()) {
+      cout << resp.Json().dump(2) << '\n';
+      rc = true;
+    }
+  }
+  return rc;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -265,18 +304,18 @@ static bool Search(Credence::Peer & peer, const Ipv4Address & addr)
       if (resp.Json().is_object()) {
         // auto  && members = resp.Json().getMemberNames();
         for (auto & [key, value] : resp.Json().items()) {
-          if ((resp.Json()[key].find("prefix") != resp.Json()[key].end())
-              && resp.Json()[key]["prefix"].is_string()) {
-            const nlohmann::json  & match = resp.Json()[key];
-            cout << key << ":\n"
-                 << setiosflags(ios::left)
-                 << "  "
-                 << setw(18) << match["prefix"].get<string>()
-                 << ' ' << setiosflags(ios::right)
-                 << setw(6) << match["daysRemaining"].get<uint32_t>() << "d "
-                 << setw(3) << match["country"].get<string>() << " ("
-                 << match["countryName"].get<string>() << ")\n"
-                 << resetiosflags(ios::right) << resetiosflags(ios::left);
+          if (value.is_array()) {
+            cout << key << ":\n";
+            for (const auto & entry : value) {
+              cout << setiosflags(ios::left)
+                   << "  "
+                   << setw(18) << entry["prefix"].get<string>()
+                   << ' ' << setiosflags(ios::right)
+                   << setw(6) << entry["daysRemaining"].get<uint32_t>() << "d "
+                   << setw(3) << entry["country"].get<string>() << " ("
+                   << entry["countryName"].get<string>() << ")\n"
+                   << resetiosflags(ios::right) << resetiosflags(ios::left);
+            }
           }
           rc = true;
         }
@@ -494,6 +533,40 @@ int main(int argc, char *argv[])
       rc = 0;
     }
   }
+  else if (string(argv[nextArg]) == "add") {
+    if (argc > ++nextArg) {
+      string  table = argv[nextArg];
+      if (argc > ++nextArg) {
+        string  country = argv[nextArg];
+        if (argc > ++nextArg) {
+          uint32_t  days = stoul(argv[nextArg], nullptr, 10);
+          if (argc > ++nextArg) {
+            vector<Ipv4Prefix>  prefixes;
+            for ( ; nextArg < argc; ++nextArg) {
+              prefixes.push_back(Ipv4Prefix(argv[nextArg]));
+            }
+            if (AddPrefixes(peer, table, country, days, prefixes)) {
+              rc = 0;
+            }
+          }
+        }
+      }
+    }
+  }
+  else if (string(argv[nextArg]) == "delete") {
+    if (argc > ++nextArg) {
+      string  table = argv[nextArg];
+      if (argc > ++nextArg) {
+        vector<Ipv4Prefix>  prefixes;
+        for ( ; nextArg < argc; ++nextArg) {
+          prefixes.push_back(Ipv4Prefix(argv[nextArg]));
+        }
+        if (DeletePrefixes(peer, table, prefixes)) {
+          rc = 0;
+        }
+      }
+    }
+  }
   else if (string(argv[nextArg]) == "activate") {
     if (argc > ++nextArg) {
       string  table = argv[nextArg];
