@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockResponder.cc 12115 2022-12-06 03:41:24Z dwm $
+// @(#) $Id: DwmMcBlockResponder.cc 12351 2024-05-03 09:12:30Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017, 2022
 //  All rights reserved.
@@ -60,7 +60,7 @@ extern "C" {
 #include "DwmMcBlockServer.hh"
 #include "DwmMcBlockTcpDrop.hh"
 
-static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/DwmMcBlockResponder.cc 12115 $");
+static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.6/apps/mcblockd/DwmMcBlockResponder.cc 12351 $");
 
 using namespace std;
 
@@ -113,6 +113,21 @@ namespace Dwm {
       return rc;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    static bool IsValidIpv4Prefix(const nlohmann::json & j)
+    {
+      if (j.is_string()) {
+        Ipv4Prefix  pfx(j.get<string>());
+        auto  rawNet = pfx.Network().Raw();
+        return ((pfx.MaskLength() > 0)
+                && (rawNet != 0)
+                && (rawNet != 0xFFFFFFFF));
+      }
+      return false;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -145,6 +160,28 @@ namespace Dwm {
               && (request.find("daysRemaining") != request.end())
               && request["daysRemaining"].is_number_unsigned());
     }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::IsValidAddRequest(const nlohmann::json & request)
+    {
+      return ((request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "add")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("country") != request.end())
+              && (request["country"].is_string())
+              && (request.find("days") != request.end())                                                                        
+              && (request["days"].is_number_unsigned())
+              && (request.find("prefixes") != request.end())
+              && (request["prefixes"].is_array())
+              && (request["prefixes"].size() > 0)
+              && std::all_of(request["prefixes"].begin(),
+                             request["prefixes"].end(),
+                             [] (auto & p) { return IsValidIpv4Prefix(p); }));
+    }
     
     //------------------------------------------------------------------------
     //!  
@@ -159,7 +196,9 @@ namespace Dwm {
               && (request.find("prefixes") != request.end())
               && (request["prefixes"].is_array())
               && (request["prefixes"].size() > 0)
-              && (request["prefixes"][0].is_string()));
+              && std::all_of(request["prefixes"].begin(),
+                             request["prefixes"].end(),
+                             [] (auto & p) { return IsValidIpv4Prefix(p); }));
     }
 
     //------------------------------------------------------------------------
@@ -175,9 +214,29 @@ namespace Dwm {
               && (request.find("prefixes") != request.end())
               && (request["prefixes"].is_array())
               && (request["prefixes"].size() > 0)
-              && (request["prefixes"][0].is_string()));
+              && std::all_of(request["prefixes"].begin(),
+                             request["prefixes"].end(),
+                             [] (auto & p) { return IsValidIpv4Prefix(p); }));
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::IsValidDeleteRequest(const nlohmann::json & request)
+    {
+      return ((request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "delete")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("prefixes") != request.end())
+              && (request["prefixes"].is_array())
+              && (request["prefixes"].size() > 0)
+              && std::all_of(request["prefixes"].begin(),
+                             request["prefixes"].end(),
+                             [] (auto & p) { return IsValidIpv4Prefix(p); }));
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -262,6 +321,99 @@ namespace Dwm {
       }
       return rc;
     }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    nlohmann::json
+    Responder::HandleAddRequest(const nlohmann::json & request)
+    {
+      Syslog(LOG_DEBUG, "request %s", request.dump().c_str());
+      nlohmann::json       rc;
+      Pf::Table  &&table =
+        _server.GetPfDevice().GetTable("", request["table"].get<string>());
+      if (table.Name() == request["table"].get<string>()) {
+        uint32_t             days = request["days"].get<uint32_t>();
+        string               country = request["country"].get<string>();
+        vector<Ipv4Prefix>   prefixes;
+        for (uint32_t i = 0; i < request["prefixes"].size(); ++i) {
+          Ipv4Prefix  prefix(request["prefixes"][i].get<string>());
+          prefixes.push_back(prefix);
+        }
+        for (auto ai = 0; ai < prefixes.size(); ++ai) {
+          Ipv4Prefix  match(prefixes[ai]);
+          if (! table.Contains(prefixes[ai], match)) {
+            if (table.Add(match)) {
+              KillSourceState(match);
+              TcpDropPrefix(match);
+              AddToDatabase(table.Name(), prefixes[ai], days, country);
+              rc["table"] = request["table"];
+              rc["prefixes"][ai]["requested"] = request["prefixes"][ai];
+              rc["prefixes"][ai]["added"] = match.ToShortString();
+            }
+            else {
+              Syslog(LOG_ERR, "Failed to add %s to %s",
+                     prefixes[ai].ToString().c_str(), table.Name().c_str());
+            }
+          }
+          else {
+            //  Entry exists in pf table.  If it's wider, just update its
+            //  'days'.  Else update country and days.
+            if (match.MaskLength() < prefixes[ai].MaskLength()) {
+              ModifyDatabaseEntry(table.Name(), match, days, "");
+            }
+            else {
+              ModifyDatabaseEntry(table.Name(), match, days, country);
+            }
+            rc["table"] = request["table"];
+            rc["prefixes"][ai]["requested"] = request["prefixes"][ai];
+            rc["prefixes"][ai]["modified"] = match.ToShortString();
+          }
+        }
+        SaveDatabase(table.Name());
+      }
+      else {
+        Syslog(LOG_ERR, "Table '%s' not found",
+               request["table"].get<string>().c_str());
+      }
+      return rc;
+    }
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    nlohmann::json
+    Responder::HandleDeleteRequest(const nlohmann::json & request)
+    {
+      nlohmann::json       rc;
+      Pf::Table  &&table =
+        _server.GetPfDevice().GetTable("", request["table"].get<string>());
+      if (table.Name() == request["table"].get<string>()) {
+        vector<Ipv4Prefix>  prefixes;
+        for (uint32_t i = 0; i < request["prefixes"].size(); ++i) {
+          Ipv4Prefix  prefix(request["prefixes"][i].get<string>());
+          prefixes.push_back(prefix);
+        }
+        rc["table"] = request["table"];
+        rc["pf_removed"] = nlohmann::json::array();
+        rc["db_removed"] = nlohmann::json::array();
+        size_t  pfi = 0, dbi = 0;
+        for (auto ai = 0; ai < prefixes.size(); ++ai) {
+          if (table.Remove(prefixes[ai])) {
+            rc["pf_removed"][pfi] = prefixes[ai].ToShortString();
+            ++pfi;
+          }
+          if (RemoveFromDatabase(table.Name(), prefixes[ai])) {
+            rc["db_removed"][dbi] = prefixes[ai].ToShortString();
+            ++dbi;
+          }
+        }
+        if (0 < dbi) {
+          SaveDatabase(table.Name());
+        }
+      }
+      return rc;
+    }
     
     //------------------------------------------------------------------------
     //!  
@@ -312,6 +464,7 @@ namespace Dwm {
             rc["prefixes"][ai]["matched"] = match.ToShortString();
           }
         }
+        SaveDatabase(table.Name());
       }
       else {
         Syslog(LOG_ERR, "Table '%s' not found",
@@ -348,6 +501,7 @@ namespace Dwm {
             }
           }
         }
+        SaveDatabase(table.Name());
       }
       return rc;
     }
@@ -413,6 +567,15 @@ namespace Dwm {
           uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
           rc["daysRemaining"] = daysLeft;
         }
+        else {
+          Syslog(LOG_ERR, "No entry found for %s in table %s",
+                 ((string)addr).c_str(),
+                 request["table"].get<string>().c_str());
+        }
+      }
+      else {
+        Syslog(LOG_ERR, "Table '%s' not found",
+               request["table"].get<string>().c_str());
       }
       return rc;
     }
@@ -447,6 +610,15 @@ namespace Dwm {
           uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
           rc["daysRemaining"] = daysLeft;
         }
+        else {
+          Syslog(LOG_ERR, "No match found for %s in table %s",
+                 pfx.ToString().c_str(),
+                 request["table"].get<string>().c_str());
+        }
+      }
+      else {
+        Syslog(LOG_ERR, "Table '%s' not found",
+               request["table"].get<string>().c_str());
       }
       return rc;
     }
@@ -461,17 +633,21 @@ namespace Dwm {
       Ipv4Address     addr(request["ipv4addr"].get<string>());
       TimeValue       now(true);
       for (const auto & db : _server.GetDatabases()) {
-        pair<Ipv4Prefix,DbEntry>  match;
-        if (db.second.FindLongest(addr, match)) {
-          if (match.second.IsActive()) {
-            rc[db.first]["prefix"] = match.second.Prefix().ToShortString();
-            rc[db.first]["country"] = match.second.Country();
-            rc[db.first]["countryName"] =
-              DbEntry::GetCountryName(match.second.Country());
-            TimeValue  timeLeft(match.second.Interval().End());
-            timeLeft -= now;
-            uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
-            rc[db.first]["daysRemaining"] = daysLeft;
+        vector<pair<Ipv4Prefix,DbEntry>>  matches;
+        if (db.second.Find(addr, matches)) {
+          size_t  n = 0;
+          for (const auto & match : matches) {
+            if (match.second.IsActive()) {
+              rc[db.first][n]["prefix"] = match.second.Prefix().ToShortString();
+              rc[db.first][n]["country"] = match.second.Country();
+              rc[db.first][n]["countryName"] =
+                DbEntry::GetCountryName(match.second.Country());
+              TimeValue  timeLeft(match.second.Interval().End());
+              timeLeft -= now;
+              uint32_t  daysLeft = (timeLeft.Secs() / (24 * 60 * 60));
+              rc[db.first][n]["daysRemaining"] = daysLeft;
+              ++n;
+            }
           }
         }
       }
@@ -527,6 +703,29 @@ namespace Dwm {
       return;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::ModifyDatabaseEntry(const string & tableName,
+                                        const Ipv4Prefix & prefix,
+                                        uint32_t days, const string & country)
+    {
+      auto  dbi = _server.GetDatabases().find(tableName);
+      if (dbi != _server.GetDatabases().end()) {
+        TimeValue  now(true);
+        TimeValue  endTime(now);
+        endTime.Set(endTime.Secs() + (days * 24 * 60 * 60), 0);
+        TimeInterval  ti(now, endTime);
+        DbEntry  dbEntry(prefix, ti, "", country);
+        if (country.empty()) {
+          dbEntry.Country(dbi->second[prefix].Country());
+        }
+        dbi->second[prefix] = dbEntry;
+        return true;
+      }
+      return false;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -542,7 +741,7 @@ namespace Dwm {
         TimeInterval  ti(now, endTime);
         DbEntry  dbEntry(prefix, ti, "", country);
         dbi->second.AddEntry(dbEntry);
-        dbi->second.Save("");
+        // dbi->second.Save("");
       }
       else {
         Syslog(LOG_ERR, "Database not found for table %s", tableName.c_str());
@@ -550,6 +749,18 @@ namespace Dwm {
       return;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    void Responder::SaveDatabase(const string & tableName)
+    {
+      auto  dbi = _server.GetDatabases().find(tableName);
+      if (dbi != _server.GetDatabases().end()) {
+        dbi->second.Save("");
+      }
+      return;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -564,24 +775,27 @@ namespace Dwm {
         }
         DbEntry  newDbEntry(prefix);
         dbi->second.AddEntry(newDbEntry);
-        dbi->second.Save("");
+        // dbi->second.Save("");
       }
       return;
     }
 
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    void Responder::RemoveFromDatabase(const string & tableName,
+    bool Responder::RemoveFromDatabase(const string & tableName,
                                        const Ipv4Prefix & prefix)
     {
+      bool  rc = false;
       auto  dbi = _server.GetDatabases().find(tableName);
       if (dbi != _server.GetDatabases().end()) {
         if (dbi->second.DeleteEntry(prefix)) {
-          dbi->second.Save("");
+          rc = true;
+          // dbi->second.Save("");
         }
       }
-      return;
+      return rc;
     }
 
     //------------------------------------------------------------------------
@@ -646,6 +860,7 @@ namespace Dwm {
                 KillSourceState(prefix);
                 TcpDropPrefix(prefix);
                 AddToDatabase(pftable.Name(), prefix, addRule.Days(), country);
+                SaveDatabase(pftable.Name());
                 Syslog(LOG_INFO, "Added %s (%s) to %s for %u days",
                        prefix.ToShortString().c_str(), country.c_str(),
                        pftable.Name().c_str(), addRule.Days());
@@ -665,6 +880,7 @@ namespace Dwm {
                     TcpDropPrefix(prefix);
                     AddToDatabase(pftable.Name(), prefix, addRule.Days(),
                                   country);
+                    SaveDatabase(pftable.Name());
                     Syslog(LOG_INFO, "Added %s (%s) to %s for %u days",
                            prefix.ToShortString().c_str(), country.c_str(),
                            pftable.Name().c_str(), addRule.Days());
@@ -714,12 +930,18 @@ namespace Dwm {
     {
       Syslog(LOG_DEBUG, "Handling request '%s'", request.dump().c_str());
       
+      if (IsValidAddRequest(request)) {
+        return HandleAddRequest(request);
+      }
       if (IsValidActivateRequest(request)) {
         return HandleActivateRequest(request);
       }
       else if (IsValidDeactivateRequest(request)) {
         return HandleDeactivateRequest(request);
       }
+      else if (IsValidDeleteRequest(request)) {
+        return HandleDeleteRequest(request);
+      }
       else if (IsValidSearchRequest(request)) {
         return HandleSearchRequest(request);
       }
